/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.io;

/**
 *  The basic thread-safe implementation of logger.
 */

public abstract class BasicLogger
implements Logger
{
  private boolean isClosed = false;
  private boolean isActive = true;

  /**
   *  Enables logging.
   */
  public synchronized void start()
  {
    isActive = true;
  }

  /**
   *  Disables logging.
   */
  public synchronized void stop()
  {
    isActive = false;
  }

  /**
   *  Performs logging action.
   *  @param s a string to log
   */
  public synchronized void log(String s)
  {
    if (!isClosed && isActive) _log(s);
  }

  /**
   *  Internal log method. Should be implemented by succeeded class.
   *  @param s a string to log
   */
  protected abstract void _log(String s);

  /**
   *  Performs any cleaning and closes a logger.
   */  
  public synchronized void close()
  {
    if (isClosed) return;
    isClosed = true;
    _close();
  }

  /**
   *  Internal close method. Should be implemented by succeeded class.
   */  
  protected abstract void _close();
}
