/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.proxy;

import java.io.*;

import org.zaval.io.*;
import org.zaval.util.*;

/**
 *  A base copier class.
 */

public abstract class ProxyCopier extends Thread
{
  private InputStream input;
  private OutputStream output;
  private OutputStream copy;
  private Logger logger;
  private Finalizable finalizer;

  /**
   *  Creates a ProxyCopier instance.
   *  @param input input stream from which data is read
   *  @param output output stream into which data is written
   *  @param copy output stream into which copy of data is written
   *  @param logger object for logging
   *  @param finalizer callback object to free allocated resources
   */
  public ProxyCopier(InputStream input, OutputStream output, OutputStream copy,
    Logger logger, Finalizable finalizer)
  {
    this.input = input;
    this.output = output;
    this.copy = copy;
    this.logger = logger;
    this.finalizer = finalizer;
  }

  /**
   *  Identifies a ProxyCopier instance.
   *  @return identifier
   */ 
  protected abstract String getId();

  /**
   *  Runs processing. Reads from input stream and writes into specified output streams.
   *  Finally executes callback method of the Finalizable object using own identifier.
   */
  public void run()
  {
    int k;
    byte[] buf = new byte[512];

    try
    {
      for(;;)
      {
        k = input.read(buf, 0, buf.length);
        logger.log(getId() + "> read " + k + " bytes");
        if (k <= 0) break;
        if (copy != null)
        {
          copy.write(buf, 0, k);
          copy.flush();
        }
        output.write(buf, 0, k);
        output.flush();
      }
    }
    catch (Throwable t)
    {
      logger.log(getId() + "> Exception: " + t.getMessage());
    }

    finalizer.free(getId());
  }
}
