/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.proxy;

import java.io.*;
import java.net.*;
import java.util.*;

import org.zaval.io.*;

/**
 *  The main running class. It reads the configuration file "proxy.ini" located
 *  in the current directory and starts specified servers.
 */

public class ProxyRunner
{
  /**
   *  Starts ProxyServer instances and the shutdown server
   *  using configuration info. If this info is invalid does nothing.
   */
  public ProxyRunner()
  {
    InputIniFile configObj;
  
    try
    {
      configObj = new InputIniFile("proxy.ini");
    }
    catch (IOException e)
    {
      System.out.println("Fatal error: can't read configuration file (proxy.ini)");
      return;
    }

    int defaultPort = configObj.getInt("default.listen.port", 8081);
    
    String serverList = configObj.getString("server.list", "");
    if (serverList.length() == 0)
    {
      System.out.println("Nothing to do. Please, specify at least one proxy in configuration file.");
      return;
    }

    Vector serverObjList = new Vector();
    
    StringTokenizer tok = new StringTokenizer(serverList, ",");
    while (tok.hasMoreTokens())
    {
      String alias = tok.nextToken().trim();
      
      int nPort = configObj.getInt(alias + ".listen.port", defaultPort);
      String serverAddress = configObj.getString(alias + ".server.address", "");
      int nServerPort = configObj.getInt(alias + ".server.port", 0);
      boolean bLogging = configObj.getBoolean(alias + ".logging", true);
      boolean bDebug = configObj.getBoolean(alias + ".debug", false);
     
      String outputDir = configObj.getString(alias + ".output.dir", "");
      if (outputDir.length() == 0) outputDir = "./";
      outputDir = fixPath(outputDir);

      try
      {      
        ProxyServer serverObj = new ProxyServer(
          alias, nPort, serverAddress, nServerPort, bLogging, bDebug, outputDir
        );
        serverObjList.add(serverObj);
        serverObj.start();
      }
      catch (SocketException se)
      {
        System.out.println("Error: can't set timeout for a socket, port " + nPort + " (" + alias + ")");
      }
      catch (IOException ioe)
      {
        System.out.println("Error: can't start proxy on port " + nPort + " (" + alias + ")");
      }
    }
    
    if (serverObjList.size() == 0) return;

    int shutdownPort = configObj.getInt("shutdown.server.port", 2001);

    try
    {    
      ShutdownServer shutdownServer = new ShutdownServer(shutdownPort, serverObjList);
      shutdownServer.start();
    }
    catch (IOException e)
    {
      System.out.println("Error: can't start shutdown server on port " + shutdownPort);
    }
  }

  /**
   *  Terminates a path string with a file separator if it is omitted.
   *  @return the fixed value
   */
  private static String fixPath(String path)
  {
    if (path == null) return null;
    String sResult = path.trim().replace('\\', '/');;
    if (sResult.length() == 0) return sResult;
    char c = sResult.charAt(sResult.length()-1);
    if (c != '/' ) sResult += "/";
    return sResult;
  }

  /**
   *  Startup method.
   */
  public static void main(String[] args)
  {
    new ProxyRunner();
  }
}
