/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.proxy;

import java.io.*;
import java.net.*;
import java.util.*;

import org.zaval.io.*;

/**
 *  The running class for shutdowning. It uses the configuration file
 *  "proxy.ini" to determine a shutdown server's listening port.
 */

public class Shutdown
{
  /**
   *  Reads the shutdown server's port from the configuration file and 
   *  connects to the server requesting the shutdown process.
   */
  public Shutdown()
  {
    InputIniFile configObj;
  
    try
    {
      configObj = new InputIniFile("proxy.ini");
    }
    catch (IOException e)
    {
      System.out.println("Fatal error: can't read configuration file (proxy.ini)");
      return;
    }
    
    int shutdownPort = configObj.getInt("shutdown.server.port", 2001);
    
    try
    {
      Socket client = new Socket("127.0.0.1", shutdownPort);
      try { client.close(); } catch (Throwable t) {}
      System.out.println("Shutdown request is sent");
    }
    catch (Throwable t)
    {
      System.out.println("Fatal error: can not connect to 127.0.0.1:" + shutdownPort);
    }
  }

  /**
   *  Startup method.
   */  
  public static void main(String[] args)
  {
    new Shutdown();
  }
}
