/**
 *     Caption: Zaval Proxy Suite
 *     $Revision: 0.43 $
 *     $Date: 2002/09/26 20:20:37 $
 *
 *     @author:     Eugene Shagun
 *     @version:    1.0
 *
 * The Zaval Proxy Suite is an easy-to-use solution that allows monitoring 
 * TCP/IP-based protocols, such as HTTP, NNTP and others. It is extremely 
 * useful in software development and can be used as a debug tool. Another 
 * area of appliance is multiple connections logging with proxy facilities. 
 * As soon as it pure java solution it can be used almost everywhere.
 * 
 * For more info on this product read Zaval Proxy Suite User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/proxy/
 * and from the SourceForge:
 * http://sourceforge.net/projects/zaval0006/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 * 
 * Copyright (C) 2000-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
package org.zaval.proxy;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 *  The ShutdownServer class is an implementation of the service server.
 *  It simply listens on the specified port and on the connection event it sets
 *  the shutdown flag of proxy servers to true. The shutdown server instance
 *  contains the list of the ProxyServer objects.
 */

public class ShutdownServer extends Thread
{
  private int nPort;
  private Vector serverObjList;
  
  private ServerSocket server;

  /**
   *  Creates an instance of the shutdown server and opens a server socket to listen.
   *  @param nPort port on which the shutdown server listens
   *  @param serverObjList list of the ProxyServer objects which should be shutdowned
   *  @throws IOException if a server socket can not be created
   */
  public ShutdownServer(int nPort, Vector serverObjList) throws IOException
  {
    this.nPort = nPort;
    this.serverObjList = serverObjList;
    
    server = new ServerSocket(nPort);
    System.out.println("Shutdown server is listening on port " + nPort);
  }

  /**
   *  Listens on the specified port. If somebody has been connected performs shutdown process
   *  and terminates its work. Each proxy server object is marked for shutdowning.
   */
  public void run()
  {
    try
    {
      Socket client = server.accept();
      try { client.close(); } catch (Throwable t) {}
      try { server.close(); } catch (Throwable t) {}
      shutdown();
    }
    catch (Throwable t) {}
  }

  /**
   *  Performs shutdown process.
   */  
  private void shutdown()
  {
    System.out.println("Shutdowning...");
    for (int i=0; i<serverObjList.size(); i++)
    {
      ProxyServer serverObj = (ProxyServer)serverObjList.get(i);
      serverObj.shutdown();
      try
      {
        Socket socket = new Socket("127.0.0.1", serverObj.getListenPort());
        socket.close();
      }
      catch (Throwable t)
      {}
    }
  }
}
